;Program Shell for any assembly language
PAGE 60,132
TITLE DISK I/O EXAMPLE
; This program
;    (1)   clears the screen,
;    (2)   reads keyboard input into the buffer,
;    (3)   writes buffer to file,
;    (4)   closes the file,
;    (5)   opens the file,
;    (6)   reads the file to buffer,
;    (7)   writes buffer to the monitor,
;    (8)   closes the file,
;    (9)   exits when user is ready
;
STSEG 	SEGMENT
      	DB 64 DUP(?)
STSEG 	ENDS
;-----------------
DTSEG 	SEGMENT
CR EQU 0DH ; DEFINING CR AS CARRIAGE RETURN CHARACTER
LF EQU 0AH ; DEFINING LF AS LINE FEED CHARACTER
HANDLE_F1 DW 0  ; FILE HANDLE - HAS TO BE 16 BITS LONG
FILE_1    DB 'C:\TASM\PROGRAMS\TRY.TXT',0   ; ASCIIZ STRING FOR FILENAME
BUFFER_1  DB 25,?, 25 DUP(0) ; FOR (2) :  MAXSIZE, COUNT, DATA AREA
BUFFER_2  DB 25 DUP(0),'$' ; FOR (7)
QUERY DB 'PLEASE ENTER A 24-CHAR SENTENCE: $' 
ERR_CREATE DB CR, LF, '**ERROR CREATING FILE**$'
ERR_OPEN DB CR, LF, '**ERROR OPENING FILE**$'
ERR_READ DB CR, LF, '**ERROR READING FILE**$'
ERR_WRITE DB CR, LF, '**ERROR WRITING FILE**$'
ERR_CLOSE DB CR, LF, '**ERROR CLOSING FILE**$'
SUCC_CREATE DB CR, LF, 'File written successfully, hit c (or C) to continue**$'
EXIT_MSG DB CR, LF, 'PRESS C (or c) KEY TO EXIT$'
DTSEG 	ENDS
;-----------------
CDSEG 	SEGMENT
MAIN	PROC FAR  ; This is the program entry point for DOS
  	ASSUME CS:CDSEG, DS:DTSEG, SS:STSEG
	MOV AX,DTSEG
	MOV DS,AX
        CALL CLEAR  ; CLEAR SCREEN
        CALL CURSOR ; SET CURSOR TO THE RIGHT PLACE
        CALL CREATE_F ; NEED TO CREATE AN OUTPUT FILE
        CALL PROMPTER ; SET UP A PROMPTER TO ENTER A STRING
        CALL READ_KB  ; WAIT FOR KEYBOARD INPUT
        CALL WRITE_F ; WRITE BUFFER TO FILE
        CALL CLOSE_F ; CLOSE THE FILE
        CALL CURSOR;
        CALL OPEN_F; OPEN FILE FOR READ
        CALL READ_F; READ FILE INTO BUFFER
        CALL CLOSE_F;       
        MOV DX, OFFSET BUFFER_2
        CALL SHOW
        MOV DX, OFFSET EXIT_MSG
        CALL SHOW
        CALL KEYPRESS
        CALL CLEAR
        MOV AH,4CH
        INT 21H
MAIN	ENDP
;--------------------------
CLEAR   PROC NEAR  
        MOV AX,0600H ; 
        MOV BH, 07
        MOV CX,0000
        MOV DX, 18F4H
        INT 10H
	RET
CLEAR   ENDP
;--------------------------
CURSOR  PROC NEAR  
        MOV AH,02 ; CHOOSE CURSOR FUNCTION 
        MOV BH, 00
        MOV DH,10  ;  ROW 10
        MOV DL, 10  ; COLUMN 10
        INT 10H
	RET
CURSOR  ENDP
;--------------------------
CREATE_F  PROC NEAR  
          MOV AH,3CH ; CREATE FILE OPTION 
          MOV CX, 0 ; WE WANT A NORMAL FILE
          MOV DX,OFFSET FILE_1  ;  DX POINTS TO STORED FILENAME
          INT 21H ; INVOKE THE FILE CREATE ROUTINE
          JC C_ERR ; IF CARRY HAS A 1, ERROR HAS OCCURRED
          MOV HANDLE_F1, AX ; IF SUCCESSFUL, AX POINTS TO FILE, SAVE TO MEM
          JMP C_EXIT
C_ERR:    MOV DX, OFFSET ERR_CREATE ; DISPLAY ERROR
          CALL SHOW
C_EXIT:   RET
CREATE_F  ENDP
;--------------------------
PROMPTER PROC NEAR  
         MOV DX, OFFSET QUERY   
         CALL SHOW 
         RET
PROMPTER ENDP
;--------------------------
READ_KB PROC   
        MOV AH,0AH
        MOV DX, OFFSET BUFFER_1
        INT 21H
	RET
READ_KB ENDP
;--------------------------
WRITE_F PROC NEAR  
        MOV AH,40H
        MOV BX, HANDLE_F1
        MOV SI, OFFSET BUFFER_1+1 ; SI POINTS TO THE DATA COUNT
        MOV CL, BYTE PTR [SI] ; CL NOW HAS THE BUFFER COUNT
        SUB CH,CH ; 
        MOV DX, OFFSET BUFFER_1+2 ; DX POINTS TO THE DATA
        INT 21H
        JNC W_SUC
        MOV DX, OFFSET ERR_WRITE
        CALL SHOW
        JMP W_EXIT
W_SUC:  MOV DX, OFFSET SUCC_CREATE
        CALL SHOW
        CALL KEYPRESS
        CALL CLEAR
W_EXIT: RET
WRITE_F ENDP
;--------------------------
CLOSE_F PROC NEAR  
        MOV AH,3EH
        MOV BX, HANDLE_F1
        INT 21H
        JNC CL_EXIT
        MOV DX, OFFSET ERR_CLOSE
        CALL SHOW
CL_EXIT: RET
CLOSE_F ENDP
;--------------------------
OPEN_F  PROC NEAR  
        MOV AH,3DH
        MOV AL, 0
        MOV DX, OFFSET FILE_1 ; TELL THE ROUTINE THE REQUIRED FILE
        INT 21H
        JC OP_ERR
        MOV HANDLE_F1, AX ; STORE THE FILE POINTER AWAY
        JMP OP_EXIT
OP_ERR: MOV DX, OFFSET ERR_OPEN
        CALL SHOW
OP_EXIT:RET
OPEN_F ENDP
;--------------------------
READ_F  PROC NEAR  
        MOV AH,3FH
        MOV BX, HANDLE_F1
        MOV SI, OFFSET BUFFER_1+1 ; SI POINTS TO THE DATA COUNT
        MOV CL, BYTE PTR [SI] ; CL NOW HAS THE BUFFER COUNT
        SUB CH,CH ; 
        MOV DX, OFFSET BUFFER_2 ; DX POINTS TO BUFFER IN MEMORY
        INT 21H
        JNC R_EXIT
        MOV DX, OFFSET ERR_READ
        CALL SHOW
R_EXIT:RET
READ_F ENDP
;--------------------------


SHOW    PROC NEAR
        MOV AH,09H   
        INT 21H
	RET
SHOW    ENDP
;--------------------------
SOUNDBELL    PROC   
        MOV AH,02 ;  
        MOV DL, 07; SENDS OUT THE BELL CHAR
        INT 21H
	RET
SOUNDBELL    ENDP
;--------------------------
KEYPRESS    PROC
AGAIN:  CALL SOUNDBELL
        MOV AH, 01 ; CHECK KEYPRESSED
        INT 16H
        JZ AGAIN ; SOUNDBELL AGAIN IF NO KEYPRESSED
        MOV AH, 0 ; CHECK WHICH CHARACTER WAS ENTERED
        INT 16H ; CHARACTER IS NOW IN AL
        CMP AL, 'c' ;
        JE EXIT
        CMP AL, 'C'
        JE EXIT
        JMP AGAIN
EXIT:   RET
KEYPRESS    ENDP

;---------------------------	

CDSEG	ENDS
	END MAIN	; THIS IS THE EXIT POINT



